
# q-expansion of newform 924.2.r.d, downloaded from the LMFDB on 23 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 924
weight = 2
poly_data = [1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[463, [1, 0, 0, 0]], [617, [1, 0, 0, 0]], [661, [1, 0, 0, 0]], [673, [0, 0, 0, -1]]]
aps_data = [[0, 0, 0, 0], [0, 0, -1, 0], [2, 0, 2, 0], [0, 0, 0, -1], [2, 1, 0, 2], [-2, 2, -2, 2], [2, 2, 2, 0], [0, 0, 0, 0], [0, 0, -3, 3], [1, -1, 0, -1], [0, -2, 2, 0], [3, -3, 0, 4], [0, 8, -4, 8], [1, 0, 3, -3], [0, -4, -2, -4], [-3, 4, -4, 3], [8, -8, 0, -10], [4, 0, 4, 0], [1, 0, 7, -7], [1, 2, 1, 0], [-2, 2, 0, 0], [7, -6, 6, -7], [-2, 8, -2, 0], [-4, 0, 2, -2], [-6, -4, 4, 6], [10, 0, 0, -10], [6, -6, 0, -12], [0, 3, -3, 3], [0, 0, 3, -3], [0, -1, -8, -1], [-1, -3, -1, 0], [-18, 0, 0, 0], [-7, -1, -7, 0], [-8, 8, 0, 6], [0, -18, 0, 0], [0, -5, 21, -5], [0, -6, 12, -6], [-16, 7, -7, 16], [2, 2, -2, -2], [0, -4, -2, -4], [0, -11, 7, -11], [-6, -14, -6, 0], [1, -1, 0, 13], [-15, 4, -15, 0], [-8, 0, 5, -5], [-18, 0, -2, 2], [-16, 9, -9, 16], [0, -18, 10, -18], [2, -2, 0, -14], [22, -22, 22, -22], [-10, 14, -14, 10], [0, 11, 2, 11], [18, 0, 18, -18], [18, -8, 8, -18], [10, -10, 0, -2], [-1, 0, 5, -5], [-2, 26, -2, 0], [-12, 12, 0, -6], [-4, 7, -7, 4], [3, 0, 3, 0], [0, -14, 2, -14], [22, -22, 0, -2], [-30, 0, -6, 6], [0, -16, 6, -16], [-6, 22, -6, 0], [-3, -12, 12, 3], [12, 0, -7, 7], [-17, 17, 0, 24], [3, -18, 3, 0], [0, -8, 8, -8], [-4, 0, 4, -4], [-15, 15, 0, 24], [6, -6, 0, -22], [18, 0, -12, 12], [-3, -17, -3, 0], [20, -4, 4, -20], [3, -3, 0, 12], [0, 0, -18, 18], [13, 7, 13, 0], [16, 0, 0, -16], [-6, 0, -14, 14], [0, 21, -21, 21], [-12, 23, -23, 12], [-2, 2, 0, 8], [-4, 0, 2, -2], [0, -9, 6, -9], [12, -1, 1, -12], [-1, 21, -1, 0], [-8, 0, -10, 10], [-5, 0, -15, 15], [-2, 24, -2, 0], [20, -8, 20, 0], [0, -13, -2, -13], [24, -24, 0, 0], [13, -13, 0, 17], [0, 32, -10, 32], [0, -22, -4, -22], [-24, 24, 0, 24], [-8, 20, -8, 0], [24, 7, -7, -24], [0, 0, 0, 0], [11, -11, 0, -19], [12, 14, -14, -12], [0, -20, -6, -20], [4, 0, 9, -9], [2, 34, 2, 0], [10, -10, 0, 22], [22, 0, 26, -26], [-1, 5, -1, 0], [-18, 18, 0, 30], [10, 2, -2, -10], [0, -3, 0, -3], [-35, 0, 5, -5], [0, 12, -2, 12], [3, -3, 0, 0], [0, 9, 0, 9], [-30, 2, -30, 0], [2, -16, 16, -2], [19, -19, 0, -12], [-36, 0, -4, 4], [-26, 0, -2, 2], [23, 16, -16, -23], [4, 0, 4, 0], [24, 0, 17, -17], [-30, 4, -4, 30], [0, -1, 45, -1], [-9, 0, -9, 0], [12, -12, 0, -6], [-12, 0, 0, 0], [18, -18, 0, 0], [-17, 2, -2, 17], [9, -42, 9, 0], [0, -27, 11, -27], [11, 0, 0, -11], [8, -34, 34, -8], [-6, 0, 10, -10], [0, 4, -36, 4], [0, -22, 0, 0], [10, -2, 10, 0], [7, 33, 7, 0], [0, -10, 34, -10], [28, -28, 0, -30], [-27, 14, -14, 27], [-24, 8, -8, 24], [-4, 4, 0, -14], [0, 2, -6, 2], [-30, 12, -30, 0], [6, 0, -12, 12], [18, 0, -12, 12], [8, 28, 8, 0], [0, -11, 20, -11], [6, 0, 30, -30], [0, 36, -20, 36], [-22, 22, 0, 34], [15, -38, 15, 0], [16, -8, 8, -16], [11, 18, 11, 0], [-30, 36, -30, 0], [12, -28, 28, -12], [38, -28, 38, 0], [20, 0, 40, -40], [9, -9, 0, -36], [-24, 0, 3, -3], [0, -16, 38, -16], [24, 9, -9, -24], [40, -40, 0, -24], [-44, 0, 4, -4], [20, -20, 0, -26]]
